Package: dcifer
Type: Package
Title: Genetic Relatedness Between Polyclonal Infections
Version: 1.5.2
Authors@R: 
    person("Inna", "Gerlovina", 
           role = c("aut", "cre"),
           email = "innager@berkeley.edu", 
           comment = c(ORCID = "0000-0002-7772-7473"))
Description: An implementation of Dcifer (Distance for complex infections: fast estimation of relatedness), an identity by descent (IBD) based method to calculate genetic relatedness between polyclonal infections from biallelic and multiallelic data. The package includes functions that format and preprocess the data, implement the method, and visualize the results. 
    Gerlovina et al. (2022) <doi:10.1093/genetics/iyac126>. 
Imports: stats, utils, graphics
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
URL: https://github.com/EPPIcenter/dcifer,
        https://eppicenter.github.io/dcifer/
Suggests: knitr, rmarkdown
VignetteBuilder: knitr
Depends: R (>= 4.4.0)
NeedsCompilation: yes
Packaged: 2026-05-06 22:25:49 UTC; igerlovina
Author: Inna Gerlovina [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-7772-7473>)
Maintainer: Inna Gerlovina <innager@berkeley.edu>
Repository: CRAN
Date/Publication: 2026-05-07 07:20:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-05-07 11:40:09 UTC; unix
Archs: dcifer.so.dSYM
