# Generated by roxygen2: do not edit by hand

export(calcAfreq)
export(formatAfreq)
export(formatDat)
export(generateReval)
export(getCOI)
export(ibdDat)
export(ibdEstM)
export(ibdPair)
export(logReval)
export(matchAfreq)
export(mixMat)
export(plotColorbar)
export(plotRel)
export(probUxUy)
export(readAfreq)
export(readDat)
useDynLib(dcifer)
